@@ -1,6 +1,6 @@
 /* mpfr_atan -- arc-tangent of a floating-point number
 
-Copyright 2001-2018 Free Software Foundation, Inc.
+Copyright 2001-2023 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #define MPFR_NEED_LONGLONG_H
@@ -56,7 +56,7 @@ static const mp_limb_t atan_table[][3] = {
   };
 
 static void
-set_table (mpfr_t y, const mp_limb_t x[3])
+set_table (mpfr_ptr y, const mp_limb_t x[3])
 {
   mpfr_prec_t p = MPFR_PREC(y);
   mp_size_t n = MPFR_PREC2LIMBS(p);
@@ -64,13 +64,14 @@ set_table (mpfr_t y, const mp_limb_t x[3])
   mp_limb_t *yp = MPFR_MANT(y);
 
   MPFR_UNSIGNED_MINUS_MODULO (sh, p);
+  MPFR_ASSERTD (n >= 1 && n <= 3);
   mpn_copyi (yp, x + 3 - n, n);
   yp[0] &= ~MPFR_LIMB_MASK(sh);
   MPFR_SET_EXP(y, 0);
 }
 #endif
 
-/* If x = p/2^r, put in y an approximation of atan(x)/x using 2^m terms
+/* If x = p/2^r, put in y an approximation to atan(x)/x using 2^m terms
    for the series expansion, with an error of at most 1 ulp.
    Assumes 0 < x < 1, thus 1 <= p < 2^r.
    More precisely, p consists of the floor(r/2) bits of the binary expansion
@@ -98,14 +99,17 @@ mpfr_atan_aux (mpfr_ptr y, mpz_ptr p, unsigned long r, int m, mpz_t *tab)
   mpfr_prec_t accu[MPFR_PREC_BITS], log2_nb_terms[MPFR_PREC_BITS];
   mpfr_prec_t precy = MPFR_PREC(y);
 
-  MPFR_ASSERTD(mpz_cmp_ui (p, 0) != 0);
-  MPFR_ASSERTD (m+1 <= MPFR_PREC_BITS);
+  MPFR_ASSERTD (mpz_sgn (p) > 0);
+  MPFR_ASSERTD (m > 0);
+  MPFR_ASSERTD (m <= MPFR_PREC_BITS - 1);
 
 #if GMP_NUMB_BITS == 64
   /* tabulate values for small precision and small value of r (which are the
      most expensive to compute) */
   if (precy <= 192)
     {
+      unsigned long u;
+
       switch (r)
         {
         case 1:
@@ -120,13 +124,15 @@ mpfr_atan_aux (mpfr_ptr y, mpz_ptr p, unsigned long r, int m, mpz_t *tab)
           return;
         case 4:
           /* p has at most 2 bits: 1 <= p <= 3 */
-          MPFR_ASSERTD(1 <= mpz_get_ui (p) && mpz_get_ui (p) <= 3);
-          set_table (y, atan_table[1 + mpz_get_ui (p)]);
+          u = mpz_get_ui (p);
+          MPFR_ASSERTD(1 <= u && u <= 3);
+          set_table (y, atan_table[1 + u]);
           return;
         case 8:
           /* p has at most 4 bits: 1 <= p <= 15 */
-          MPFR_ASSERTD(1 <= mpz_get_ui (p) && mpz_get_ui (p) <= 15);
-          set_table (y, atan_table[4 + mpz_get_ui (p)]);
+          u = mpz_get_ui (p);
+          MPFR_ASSERTD(1 <= u && u <= 15);
+          set_table (y, atan_table[4 + u]);
           return;
         }
     }
@@ -150,10 +156,11 @@ mpfr_atan_aux (mpfr_ptr y, mpz_ptr p, unsigned long r, int m, mpz_t *tab)
       MPFR_ASSERTD (r > n);
       r -= n;
     }
-  /* since |p/2^r| < 1, and p is a non-zero integer, necessarily r > 0 */
 
+  /* Since |p/2^r| < 1, and p is a non-zero integer, necessarily r > 0. */
   MPFR_ASSERTD (mpz_sgn (p) > 0);
   MPFR_ASSERTD (m > 0);
+  MPFR_ASSERTD (r > 0);
 
   /* check if p=1 (special case) */
   l = 0;
@@ -177,7 +184,7 @@ mpfr_atan_aux (mpfr_ptr y, mpz_ptr p, unsigned long r, int m, mpz_t *tab)
       /* main loop */
       n = 1UL << m;
       MPFR_ASSERTN (n != 0);  /* no overflow */
-      /* the ith term being X^i/(2i+1) with X=p/2^r, we can stop when
+      /* the i-th term being X^i/(2i+1) with X=p/2^r, we can stop when
          p^i/2^(r*i) < 2^(-precy), i.e. r*i > precy + log2(p^i) */
       for (i = k = done = 0; (i < n) && (done == 0); i += 2, k ++)
         {
@@ -209,7 +216,7 @@ mpfr_atan_aux (mpfr_ptr y, mpz_ptr p, unsigned long r, int m, mpz_t *tab)
             }
         }
     }
-  else /* special case p=1: the ith term being X^i/(2i+1) with X=1/2^r,
+  else /* special case p=1: the i-th term being X^i/(2i+1) with X=1/2^r,
           we can stop when r*i > precy i.e. i > precy/r */
     {
       n = 1UL << m;
@@ -284,15 +291,15 @@ mpfr_atan (mpfr_ptr atan, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
   mpfr_exp_t exptol;
   mpfr_prec_t prec, realprec, est_lost, lost;
   unsigned long twopoweri, log2p, red;
-  int comparaison, inexact;
+  int comparison, inexact;
   int i, n0, oldn0;
   MPFR_GROUP_DECL (group);
   MPFR_SAVE_EXPO_DECL (expo);
   MPFR_ZIV_DECL (loop);
 
   MPFR_LOG_FUNC
-    (("x[%Pu]=%.*Rg rnd=%d", mpfr_get_prec (x), mpfr_log_prec, x, rnd_mode),
-     ("atan[%Pu]=%.*Rg inexact=%d",
+    (("x[%Pd]=%.*Rg rnd=%d", mpfr_get_prec (x), mpfr_log_prec, x, rnd_mode),
+     ("atan[%Pd]=%.*Rg inexact=%d",
       mpfr_get_prec (atan), mpfr_log_prec, atan, inexact));
 
   /* Singular cases */
@@ -339,8 +346,8 @@ mpfr_atan (mpfr_ptr atan, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
   MPFR_SAVE_EXPO_MARK (expo);
 
   /* Other simple case arctan(-+1)=-+pi/4 */
-  comparaison = mpfr_cmp_ui (xp, 1);
-  if (MPFR_UNLIKELY (comparaison == 0))
+  comparison = mpfr_cmp_ui (xp, 1);
+  if (MPFR_UNLIKELY (comparison == 0))
     {
       int neg = MPFR_IS_NEG (x);
       inexact = mpfr_const_pi (atan, MPFR_IS_POS (x) ? rnd_mode
@@ -401,7 +408,7 @@ mpfr_atan (mpfr_ptr atan, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
          MPFR_SAVE_EXPO_MARK, but let's check that for maintainability. */
       MPFR_ASSERTD (__gmpfr_emax <= 1 - __gmpfr_emin);
 
-      if (comparaison > 0) /* use atan(xp) = Pi/2 - atan(1/xp) */
+      if (comparison > 0) /* use atan(xp) = Pi/2 - atan(1/xp) */
         mpfr_ui_div (sk, 1, xp, MPFR_RNDN);
       else
         mpfr_set (sk, xp, MPFR_RNDN);
@@ -418,7 +425,7 @@ mpfr_atan (mpfr_ptr atan, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
           mpfr_add_ui (tmp, tmp, 1, MPFR_RNDN);
           mpfr_sqrt (tmp, tmp, MPFR_RNDN);
           mpfr_sub_ui (tmp, tmp, 1, MPFR_RNDN);
-          if (red == 0 && comparaison > 0)
+          if (red == 0 && comparison > 0)
             /* use xp = 1/sk */
             mpfr_mul (sk, tmp, xp, MPFR_RNDN);
           else
@@ -486,9 +493,9 @@ mpfr_atan (mpfr_ptr atan, mpfr_srcptr x, mpfr_rnd_t rnd_mode)
       mpfr_add (arctgt, arctgt, sk, MPFR_RNDN);
 
       /* argument reduction */
-      mpfr_mul_2exp (arctgt, arctgt, red, MPFR_RNDN);
+      mpfr_mul_2ui (arctgt, arctgt, red, MPFR_RNDN);
 
-      if (comparaison > 0)
+      if (comparison > 0)
         { /* atan(x) = Pi/2-atan(1/x) for x > 0 */
           mpfr_const_pi (tmp, MPFR_RNDN);
           mpfr_div_2ui (tmp, tmp, 1, MPFR_RNDN);
